% Vertex-Energy Distributions, cite:
% LJ. Stankovi?, E. Sejdi?, and M. Dakovi?, ?Vertex-Frequency Energy Distributions,?
% IEEE Signal Processing Letters, accepted, 2017.
%


clear, close all

load graph_definition
N=length(xx);

kapa=0.25;
W=zeros(N);

for i=1:N
    for j=i+1:N
        dij=sqrt((xx(i)-xx(j))^2+(yy(i)-yy(j))^2);
        if dij<kapa
            W(i,j)=exp(-(dij/(0.7*kapa)).^2);
            W(j,i)=exp(-(dij/(0.7*kapa)).^2);
        end
    end
end

SetFigureDefaults(7.8,16)
gr11=subplot(2,1,1);
hold on
for i=1:N
    for j=i+1:N
        if W(i,j)>0
            h=plot([xx(i),xx(j)],[yy(i),yy(j)],'b-');
            set(h,'Color',[0.7,0.7,1])
        end
    end
end
h=plot(xx,yy,'k.');
hold off

dr=0.015;
for i=1:N
    hr(i)=rectangle('Position',[xx(i)-dr,yy(i)-dr,2*dr,2*dr],'Curvature',1);
    set(hr(i),'FaceColor','w')
    ht(i)=text(xx(i),yy(i),num2str(i),'fontsize',4,'horizontalAlignment','center');
end

axis off
axis equal
colormap(hot)

D=diag(sum(W));
L=D-W;
[U,lambda]=eig(L);

% window (smaller wT - wider window)
wT=0.2;

% Signal definition
x1=U(:,51);   o1=33:64;
x2=U(:,9);  o2=1:33;
x3=U(:,13);  o3=32:64;
x4=U(:,23);  o4=20:40;
x5=U(:,27);  o5=16:48;
x6=U(:,42);  o6=33:64;
x7=U(:,3);   o7=1:64;
x8=U(:,54);  o8=1:33;
x9=U(:,59);  o9=34:64;
x10=U(:,17);  o10=40:64;

x=zeros(N,1);
x(o1)=x(o1)+.8*x1(o1);
x(o2)=x(o2)+2*x2(o2);
x(o3)=x(o3)+x3(o3);
x(o4)=x(o4)+.7*x4(o4);
x(o5)=x(o5)+.8*x5(o5);
x(o6)=x(o6)+x6(o6);
x(o7)=x(o7)+x7(o7);
x(o8)=x(o8)+2*x8(o8);
x(o9)=x(o9)+1.5*x9(o9);
x(o10)=x(o10)+x10(o10);

%    Window defined using the vertex neighborhood
%    as in:
%    LJ. Stankovi?, M. Dakovi?, and E. Sejdi?,
%    ?Vertex-Frequency Analysis: A Way to Localize Graph Spectral Components,?
%    IEEE Signal Processing Magazine, Vol.34, No. 4, July 2017, pp. 176-182
%
%    wN=zeros(N,1);
%    VF=zeros(N,N);
%     for wc=1:N
%         Nw=[wc];
%         WW=W;
%         while 1
%             N1=find(WW(wc,:)>wT);
%             N1=setdiff(N1,Nw);
%             if isempty(N1)
%                 break
%             end
%             Nw=[Nw,N1];
%             WW=MatMul(WW,W);
%         end
%         wN(wc)=length(Nw);
%         wf=zeros(N,1);
%         wf(Nw)=1;
%         xw=x.*wf;
%         VF(:,wc)=U.'*xw;
%         if wc==1
%             set(hr(Nw),'FaceColor',[1,0.8,0.8])
%             set(hr(wc),'FaceColor',[0.7,1,0.7])
%             set(hr(wc),'LineWidth',1)
%         end
%     end
%
%     subplot(4,1,2+2)
%     imagesc(1:N,1:N,abs(VF(:,:)').^2)
%     shading interp
%     xlabel('$k$','interpreter','latex')
%     ylabel('$n$','interpreter','latex')
%     axis xy
%     colormap(hot)



gr22=subplot(8,1,2+3);
stem(x,'.')
axis([0,65,1.2*min(x),1.2*max(x)])
xlabel('$n$','interpreter','latex')
ylabel('$x(n)$','interpreter','latex')
a=get(gr22,'Position');
a(1)=a(1)-0.00;
a(2)=a(2)+0.02;
set(gr22,'Position',a);

subplot(8,1,2+4)
X=U.'*x;
stem(abs(X),'.')
axis([0,65,-0.1,1.2*max(abs(X))])
xlabel('$k$','interpreter','latex')
ylabel('$|X(\lambda_k)|$','interpreter','latex')


% LVS
tau=2;
lam=diag(lambda);
H=exp(-lam*tau);
VF1=zeros(N,N);
for m=1:N
    h=0;
    for k=1:N
        h=h+H(k)*U(:,k)*U(m,k);
    end
    h=h*0.6;
    xw=x.*h;
    VF1(:,m)=U.'*xw;
end

subplot(4,1,2+2)
colormap(hot)
imagesc(1:N,1:N,abs(VF1(:,:)'))
xlabel('$k$','interpreter','latex')
ylabel('$n$','interpreter','latex')
axis xy
a=get(gr11,'Position');
a(1)=a(1)-0.00;
a(2)=a(2)-0.04;
set(gr11,'Position',a);

figure(3)
VF_Marg_LVS(abs(VF1').^2)
colormap(hot)

X=U'*x;
[Jk Nk]=size(U);
E=0;


for jk=1:Jk
    RR(jk)=0
    for nk=1:Nk
        RR(jk)=RR(jk)+W(jk,nk)*(x(jk)-x(nk))^2;
        P(jk,nk)=0;
        for ii=1:Jk
            if nk==1, E=E+W(jk,ii)*(x(jk)-x(ii))^2/2; end
            
            P(jk,nk)=P(jk,nk)+W(jk,ii)*(x(jk)-x(ii))*X(nk)*(U(jk,nk)-U(ii,nk));
            
        end
    end
end

figure(4)
VF_Marg_D(P)
colormap(hot)


for jk=1:Jk
    for nk=1:Nk
        E(jk,nk)=x(jk)*X(nk)*U(jk,nk);
    end
end


figure(5)
VF_Marg(E)
colormap(hot)



figure(2)
SetFigureDefaults(7.8,4.1)
LL=diag(lambda)+0.00000001;
X_DD=LL.*X;
X_II=X./LL;
X_II(1)=0;

xD=U*X_DD;
xI=U*X_II;

gr22=subplot(2,1,1);
stem(xD,'.')
axis([0,65,1.3*min(xD),1.3*max(xD)])
xlabel('$n$','interpreter','latex')
ylabel('$\mathcal{L}_{x}(n)$','interpreter','latex')
a=get(gr22,'Position');
a(1)=a(1)-0.00;
a(2)=a(2)+0.02;
set(gr22,'Position',a);

subplot(2,1,2)
stem(xI,'.')
axis([0,65,1.3*min(xI),1.3*max(xI)])
xlabel('$n$','interpreter','latex')
ylabel('$\mathcal{L}^{-1}_{x}(n)$','interpreter','latex')


